﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

using Topics.Radical.ComponentModel;
using Topics.Radical.ComponentModel.Messaging;
using Topics.Radical.Linq;
using Topics.Radical.Messaging;
using Topics.Radical.Windows.Presentation.Boot;

namespace Topics.Radical.Windows.Presentation.Puzzle.Boot.Installers
{
    public class DefaultDescriptor : IPuzzleSetupDescriptor
    {
        #region IPuzzleSetupDescriptor

        public void Setup(IPuzzleContainer container, Func<IEnumerable<Type>> knownTypesProvider)
        {
            var conventions = container.Resolve<BootstrapConventions>();
            var allTypes = knownTypesProvider();

            allTypes.Where(t => conventions.IsMessageHandler(t) && !conventions.IsExcluded(t))
                .Select(t => new
                {
                    Contracts = conventions.SelectMessageHandlerContracts(t),
                    Implementation = t
                })
                .ForEach(descriptor =>
                {
                    container.Register(
                        EntryBuilder.For(descriptor.Contracts.First())
                            .ImplementedBy(descriptor.Implementation)
                    );
                });

            container.Register(
                EntryBuilder.For<Application>()
                    .UsingFactory(() => Application.Current)
                    .WithLifestyle(Lifestyle.Singleton)
            );

            container.Register(
                EntryBuilder.For<IMessageBroker>()
                    .ImplementedBy<MessageBroker>()
                    .WithLifestyle(Lifestyle.Singleton)
                    //.Overridable()
            );
        }

        #endregion
    }
}
