﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using Topics.Radical.ComponentModel.Messaging;
using Topics.Radical.Windows.Presentation.Messaging;
using Windows.ApplicationModel.Activation;
using Windows.ApplicationModel.Core;

namespace Topics.Radical.Windows.Presentation.Boot
{
	public abstract  class ApplicationBootstrapper : IServiceProvider
	{
		bool isAutoBootEnabled = true;

		IServiceProvider serviceProvider;
		Type homeViewType;

		Action<Boot.BootstrapConventions> onBeforeInstall;

		protected ApplicationBootstrapper()
		{

		}

		public void Boot()
		{
			if (!this.isAutoBootEnabled)
			{
				//this.InitializeCoreComponents(args);
				//this.OnBoot(args);
				this.OnBoot();
			}
		}

		protected abstract IServiceProvider CreateServiceProvider();

		public void DefineHomeAs<THomeViewType>()
		{
			this.DefineHomeAs(typeof(THomeViewType));
		}

		public void DefineHomeAs(Type homeViewType)
		{
			this.homeViewType = homeViewType;
		}

		public void DisableAutoBoot()
		{
			this.isAutoBootEnabled = false;
		}

		public ApplicationBootstrapper OnBeforeInstall(Action<Boot.BootstrapConventions> onBeforeInstall)
		{
			this.onBeforeInstall = onBeforeInstall;

			return this;
		}

		void OnBoot()
		{
			this.serviceProvider = this.CreateServiceProvider();

			if (this.onBeforeInstall != null)
			{
				var conventions = this.serviceProvider.GetService<Boot.BootstrapConventions>();
				this.onBeforeInstall(conventions);
			}

			//this.catalog = this.CreateAggregateCatalog(this.serviceProvider);
			//this.compositionContainer = this.CreateCompositionContainer(this.catalog, this.serviceProvider);

			//this.compositionContainer.ComposeParts(this);

			//this.OnCompositionContainerComposed(this.compositionContainer, this.serviceProvider);
			//this.SetupUICompositionEngine(this.serviceProvider);

			//this.InitializeCurrentPrincipal();
			//this.InitializeCultures();

			//this.OnBoot(this.serviceProvider);

			//if (!this.IsShuttingDown)
			//{
			//	this.OnBootCompleted(this.serviceProvider);
			//	this.isBootCompleted = true;
			//}
		}

		#region IServiceProvider

		public object GetService(Type serviceType)
		{
			return this.serviceProvider.GetService(serviceType);
		}

		#endregion
	}
}
